/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2007-2012 Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
package org.sonatype.nexus.capabilities.client.rest;

import static com.google.common.base.Preconditions.checkNotNull;

import java.util.Set;
import javax.inject.Inject;
import javax.inject.Named;
import javax.inject.Singleton;

import org.sonatype.nexus.capabilities.client.Capabilities;
import org.sonatype.nexus.capabilities.client.internal.JerseyCapabilities;
import org.sonatype.nexus.capabilities.client.spi.JerseyCapabilityFactory;
import org.sonatype.nexus.client.core.Condition;
import org.sonatype.nexus.client.core.condition.NexusStatusConditions;
import org.sonatype.nexus.client.core.spi.SubsystemFactory;
import org.sonatype.nexus.client.rest.jersey.JerseyNexusClient;
import com.google.common.collect.Sets;

/**
 * Jersey based Capabilities Nexus Client Subsystem factory.
 *
 * @since 2.1
 */
@Named
@Singleton
public class JerseyCapabilitiesSubsystemFactory
    implements SubsystemFactory<Capabilities, JerseyNexusClient>
{

    private final Set<JerseyCapabilityFactory> capabilityFactories;

    public JerseyCapabilitiesSubsystemFactory()
    {
        this.capabilityFactories = Sets.newHashSet();
    }

    @Inject
    public JerseyCapabilitiesSubsystemFactory( final Set<JerseyCapabilityFactory> capabilityFactories )
    {
        this.capabilityFactories = checkNotNull( capabilityFactories );
    }

    @Override
    public Condition availableWhen()
    {
        return NexusStatusConditions.any20AndLater();
    }

    @Override
    public Class<Capabilities> getType()
    {
        return Capabilities.class;
    }

    @Override
    public Capabilities create( final JerseyNexusClient nexusClient )
    {
        return new JerseyCapabilities( nexusClient, capabilityFactories );
    }

}
